# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

import logging
from ansys.meshing.prime.internals import utils

class Model(CoreObject, CommunicationManager):
    """Model is the nucleus of Prime. Model forms the base and contains all the information about Prime.

    You can access any information in Prime only through Model.
    Model allows you to query TopoData, ControlData, Parts, SizeFields and more.

    Parameters
    ----------
    comm : Communicator
        Communicator to connect with the Ansys Prime server.
    id : int
        Id of the Model.
    object_id : int
        Object id of the Model.
    name : str
        Name of the Model.
    """

    def __init__(self, comm, id: int, object_id: int, name: str):
        """ Initialize Model """
        self._comm = comm
        self._logger = logging.getLogger("PyPrimeMesh")
        self._id = id
        self._object_id = object_id
        self._name = name

    def _print_logs_before_command(self, command, args):
        utils.print_logs_before_command(self._logger, command, args)

    def _print_logs_after_command(self, command, args = None):
        utils.print_logs_after_command(self._logger, command, args)

    def _print_beta_api_warning(self, command):
        utils.print_beta_api_warning(self._logger, command)

    def delete_parts(self, part_ids : Iterable[int]) -> DeleteResults:
        """ Delete the parts and its entities.


        Parameters
        ----------
        part_ids : Iterable[int]
            Ids of parts to be deleted.

        Returns
        -------
        DeleteResults
            Return the DeleteResults.


        Examples
        --------
        >>> results = model.delete_parts(part_ids)

        """
        if not isinstance(part_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'part_ids'. Valid argument type is Iterable[int].")
        args = {"part_ids" : part_ids}
        command_name = "PrimeMesh::Model/DeleteParts"
        self._print_logs_before_command("delete_parts", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("delete_parts", DeleteResults(model = self, json_data = result))
        return DeleteResults(model = self, json_data = result)

    def merge_parts(self, part_ids : Iterable[int], params : MergePartsParams) -> MergePartsResults:
        """ Merge given parts into one.


        Parameters
        ----------
        part_ids : Iterable[int]
            Ids of parts to be merged.
        params : MergePartsParams
            Parameters to merge parts.

        Returns
        -------
        MergePartsResults
            Returns the MergePartsResults.


        Examples
        --------
        >>> params = prime.MergePartsParams(model = model)
        >>> results = model.merge_parts(part_ids, params)

        """
        if not isinstance(part_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'part_ids'. Valid argument type is Iterable[int].")
        if not isinstance(params, MergePartsParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is MergePartsParams.")
        args = {"part_ids" : part_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Model/MergeParts"
        self._print_logs_before_command("merge_parts", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("merge_parts", MergePartsResults(model = self, json_data = result))
        return MergePartsResults(model = self, json_data = result)

    def set_global_sizing_params(self, params : GlobalSizingParams) -> SetSizingResults:
        """ Set the global sizing parameters to initialize surfer parameters and various size control parameters.


        Parameters
        ----------
        params : GlobalSizingParams
            Global sizing parameters.

        Examples
        --------
        >>> model = client.model
        >>> model.set_global_sizing_params(
        >>>           prime.GlobalSizingParams(model=model,
        >>>           min = 0.1, max = 1.0, growth_rate = 1.2))

        """
        if not isinstance(params, GlobalSizingParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is GlobalSizingParams.")
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::Model/SetGlobalSizingParams"
        self._print_logs_before_command("set_global_sizing_params", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("set_global_sizing_params", SetSizingResults(model = self, json_data = result))
        return SetSizingResults(model = self, json_data = result)

    def get_active_volumetric_size_fields(self) -> Iterable[int]:
        """ Get the active sizefield ids.


        Returns
        -------
        Iterable[int]
            Return the list of active sizefield ids.


        Examples
        --------
        >>> model = client.model
        >>> active_size_field_ids = model.get_active_volumetric_size_fields()

        """
        args = {}
        command_name = "PrimeMesh::Model/GetActiveVolumetricSizeFields"
        self._print_logs_before_command("get_active_volumetric_size_fields", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_active_volumetric_size_fields")
        return result

    def get_volumetric_size_fields(self) -> Iterable[int]:
        """ Get the sizefield ids.


        Returns
        -------
        Iterable[int]
            Return the list of sizefield ids.


        Examples
        --------
        >>> model = client.model
        >>> size_field_ids = model.get_volumetric_size_fields()

        """
        args = {}
        command_name = "PrimeMesh::Model/GetVolumetricSizeFields"
        self._print_logs_before_command("get_volumetric_size_fields", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_volumetric_size_fields")
        return result

    def activate_volumetric_size_fields(self, size_field_ids : Iterable[int]):
        """ Activate the sizefields identified by the given sizefield ids.


        Parameters
        ----------
        size_field_ids : Iterable[int]
            List of sizefield ids.

        Examples
        --------
        >>> model = client.model
        >>> model.activate_volumetric_size_fields(size_field_ids)

        """
        if not isinstance(size_field_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'size_field_ids'. Valid argument type is Iterable[int].")
        args = {"size_field_ids" : size_field_ids}
        command_name = "PrimeMesh::Model/ActivateVolumetricSizeFields"
        self._print_logs_before_command("activate_volumetric_size_fields", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("activate_volumetric_size_fields")

    def deactivate_volumetric_size_fields(self, size_field_ids : Iterable[int]):
        """ Deactivate the size fields with the given size field ids.


        Parameters
        ----------
        size_field_ids : Iterable[int]
            List of sizefield ids.

        Examples
        --------
        >>> model = client.model
        >>> model.deactivate_volumetric_size_fields(size_field_ids)

        """
        if not isinstance(size_field_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'size_field_ids'. Valid argument type is Iterable[int].")
        args = {"size_field_ids" : size_field_ids}
        command_name = "PrimeMesh::Model/DeactivateVolumetricSizeFields"
        self._print_logs_before_command("deactivate_volumetric_size_fields", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("deactivate_volumetric_size_fields")

    def delete_volumetric_size_fields(self, size_field_ids : Iterable[int]):
        """ Delete the sizefields identified by the given sizefield ids.


        Parameters
        ----------
        size_field_ids : Iterable[int]
            List of sizefield ids.

        Examples
        --------
        >>> model = client.model
        >>> model.delete_volumetric_size_fields(size_field_ids)

        """
        if not isinstance(size_field_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'size_field_ids'. Valid argument type is Iterable[int].")
        args = {"size_field_ids" : size_field_ids}
        command_name = "PrimeMesh::Model/DeleteVolumetricSizeFields"
        self._print_logs_before_command("delete_volumetric_size_fields", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("delete_volumetric_size_fields")

    def set_suggested_size_field_name(self, size_field_id : int, name : str) -> SetNameResults:
        """ Sets the suggested name of size field with the given id.


        Parameters
        ----------
        size_field_id : int
            Size field id.
        name : str
            Name of the size field.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> model = client.model
        >>> model.set_suggested_size_field_name(size_field_id, name)

        """
        if not isinstance(size_field_id, int):
            raise TypeError("Invalid argument type passed for 'size_field_id'. Valid argument type is int.")
        if not isinstance(name, str):
            raise TypeError("Invalid argument type passed for 'name'. Valid argument type is str.")
        args = {"size_field_id" : size_field_id,
        "name" : name}
        command_name = "PrimeMesh::Model/SetSuggestedSizeFieldName"
        self._print_beta_api_warning("set_suggested_size_field_name")
        self._print_logs_before_command("set_suggested_size_field_name", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("set_suggested_size_field_name", SetNameResults(model = self, json_data = result))
        return SetNameResults(model = self, json_data = result)

    def get_size_field_name(self, size_field_id : int) -> str:
        """ Gets the name of size field with the given id.


        Parameters
        ----------
        size_field_id : int
            Size field id.

        Returns
        -------
        str
            Returns the name of the size field.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> model = client.model
        >>> model.get_size_field_name(size_field_id)

        """
        if not isinstance(size_field_id, int):
            raise TypeError("Invalid argument type passed for 'size_field_id'. Valid argument type is int.")
        args = {"size_field_id" : size_field_id}
        command_name = "PrimeMesh::Model/GetSizeFieldName"
        self._print_beta_api_warning("get_size_field_name")
        self._print_logs_before_command("get_size_field_name", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_size_field_name")
        return result

    def set_num_threads(self, num : int):
        """ Sets the number of threads for multithreaded operation.


        Parameters
        ----------
        num : int
            Number of threads.

        Examples
        --------
        >>> model = client.model
        >>> model.set_num_threads(4)

        """
        if not isinstance(num, int):
            raise TypeError("Invalid argument type passed for 'num'. Valid argument type is int.")
        args = {"num" : num}
        command_name = "PrimeMesh::Model/SetNumThreads"
        self._print_logs_before_command("set_num_threads", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("set_num_threads")

    def get_num_threads(self) -> int:
        """ Get the number of threads enabled for multithreaded operation.


        Returns
        -------
        int
            Returns the number of threads.


        Examples
        --------
        >>> model = client.model
        >>> num_threads = model.get_num_threads()

        """
        args = {}
        command_name = "PrimeMesh::Model/GetNumThreads"
        self._print_logs_before_command("get_num_threads", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_num_threads")
        return result

    def get_num_compute_nodes(self) -> int:
        """ Get the number of compute nodes.


        Returns
        -------
        int
            Returns the number of compute nodes.


        Examples
        --------
        >>> model = client.model
        >>> num_compute_nodes = model.get_num_compute_nodes()

        """
        args = {}
        command_name = "PrimeMesh::Model/GetNumComputeNodes"
        self._print_logs_before_command("get_num_compute_nodes", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_num_compute_nodes")
        return result

    def start_distributed_meshing(self):
        """ Enables distributed meshing mode.


        Examples
        --------
        >>> client = prime.launch_prime()
        >>> model = client.model
        >>> model.start_distributed_meshing()

        """
        args = {}
        command_name = "PrimeMesh::Model/StartDistributedMeshing"
        self._print_logs_before_command("start_distributed_meshing", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("start_distributed_meshing")

    def create_zone(self, suggested_name : str, type : ZoneType) -> CreateZoneResults:
        """ Creates zone for the given zone type.


        Parameters
        ----------
        suggested_name : str
            Suggested name for the zone to be created.
        type : ZoneType
            Type of the zone to be created.

        Returns
        -------
        CreateZoneResults
            Returns the CreateZoneResults.


        Examples
        --------
        >>> client = prime.launch_prime()
        >>> model = client.model
        >>> results = model.create_zone("wall", prime.ZoneType.FACE)

        """
        if not isinstance(suggested_name, str):
            raise TypeError("Invalid argument type passed for 'suggested_name'. Valid argument type is str.")
        if not isinstance(type, ZoneType):
            raise TypeError("Invalid argument type passed for 'type'. Valid argument type is ZoneType.")
        args = {"suggested_name" : suggested_name,
        "type" : type}
        command_name = "PrimeMesh::Model/CreateZone"
        self._print_logs_before_command("create_zone", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("create_zone", CreateZoneResults(model = self, json_data = result))
        return CreateZoneResults(model = self, json_data = result)

    def delete_zone(self, zone_id : int) -> DeleteZoneResults:
        """ Deletes zone with the given id.


        Parameters
        ----------
        zone_id : int
            Id of the zone to be deleted.

        Returns
        -------
        DeleteZoneResults
            Returns the DeleteZoneResults.


        Examples
        --------
        >>> client = prime.launch_prime()
        >>> model = client.model
        >>> results = model.delete_zone(1)

        """
        if not isinstance(zone_id, int):
            raise TypeError("Invalid argument type passed for 'zone_id'. Valid argument type is int.")
        args = {"zone_id" : zone_id}
        command_name = "PrimeMesh::Model/DeleteZone"
        self._print_logs_before_command("delete_zone", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("delete_zone", DeleteZoneResults(model = self, json_data = result))
        return DeleteZoneResults(model = self, json_data = result)

    def get_zone_by_name(self, zone_name : str) -> int:
        """ Gets the zone with the provided name.


        Parameters
        ----------
        zone_name : str
            Name of the zone.

        Returns
        -------
        int
            Returns id of the zone.


        Examples
        --------
        >>> client = prime.launch_prime()
        >>> model = client.model
        >>> zone_id = model.get_zone_by_name("inlet")

        """
        if not isinstance(zone_name, str):
            raise TypeError("Invalid argument type passed for 'zone_name'. Valid argument type is str.")
        args = {"zone_name" : zone_name}
        command_name = "PrimeMesh::Model/GetZoneByName"
        self._print_logs_before_command("get_zone_by_name", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_zone_by_name")
        return result

    def get_zone_name(self, id : int) -> str:
        """ Get the name of given zone.


        Parameters
        ----------
        id : int
            Id of zone.

        Returns
        -------
        str
            Return the zone name. Return empty name if the id is invalid.


        Examples
        --------
        >>> client = prime.launch_prime()
        >>> model = client.model
        >>> name = model.get_zone_name(id)

        """
        if not isinstance(id, int):
            raise TypeError("Invalid argument type passed for 'id'. Valid argument type is int.")
        args = {"id" : id}
        command_name = "PrimeMesh::Model/GetZoneName"
        self._print_logs_before_command("get_zone_name", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_zone_name")
        return result

    def set_suggested_zone_name(self, id : int, name : str) -> SetNameResults:
        """ Sets the unique name for zone with given id based on the suggested name.


        Parameters
        ----------
        id : int
            Id of the zone to set suggested name.
        name : str
            Suggested name for the zone.

        Returns
        -------
        SetNameResults
            Returns the SetNameResults. results.assignedName indicates the assigned name to the zone.


        Examples
        --------
        >>> client = prime.launch_prime()
        >>> model = client.model
        >>> results = model.set_suggested_zone_name(id = 5, name = "zone1")

        """
        if not isinstance(id, int):
            raise TypeError("Invalid argument type passed for 'id'. Valid argument type is int.")
        if not isinstance(name, str):
            raise TypeError("Invalid argument type passed for 'name'. Valid argument type is str.")
        args = {"id" : id,
        "name" : name}
        command_name = "PrimeMesh::Model/SetSuggestedZoneName"
        self._print_logs_before_command("set_suggested_zone_name", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("set_suggested_zone_name", SetNameResults(model = self, json_data = result))
        return SetNameResults(model = self, json_data = result)

    def set_working_directory(self, path : str):
        """ Set working directory.

        Set working directory to be considered for file i/o when file paths are relative.

        Parameters
        ----------
        path : str
            Path to the directory.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> client = prime.launch_prime()
        >>> model = client.model
        >>> zones = model.set_working_directory("C:/input_files")

        """
        if not isinstance(path, str):
            raise TypeError("Invalid argument type passed for 'path'. Valid argument type is str.")
        args = {"path" : path}
        command_name = "PrimeMesh::Model/SetWorkingDirectory"
        self._print_beta_api_warning("set_working_directory")
        self._print_logs_before_command("set_working_directory", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("set_working_directory")

    @property
    def id(self):
        """ Get the id of Model."""
        return self._id

    @property
    def name(self):
        """ Get the name of Model."""
        return self._name



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class HexToSpline(CoreObject):
    """Converts all-hex mesh to spline.

    Parameters
    ----------
    model : Model
        Server model to create HexToSpline object.
    """

    def __init__(self, model: CommunicationManager):
        """ Initialize HexToSpline """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::HexToSpline/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for HexToSpline. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for HexToSpline. """
        command_name = "PrimeMesh::HexToSpline/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def convert_hex_to_spline(self, input_scope : ScopeDefinition, hex_to_spline_params : HexToSplineParams) -> IGAResults:
        """ Converts fully hex mesh with topology to spline.


        Parameters
        ----------
        input_scope : ScopeDefinition
            Scope definition for input hex mesh.
        hex_to_spline_params : HexToSplineParams
            Parameters to convert hex to spline.

        Returns
        -------
        IGAResults
            Returns the IGAResults structure.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = hexToSpline.ConvertHexToSpline(input_scope, hex_to_spline_params)

        """
        if not isinstance(input_scope, ScopeDefinition):
            raise TypeError("Invalid argument type passed for 'input_scope'. Valid argument type is ScopeDefinition.")
        if not isinstance(hex_to_spline_params, HexToSplineParams):
            raise TypeError("Invalid argument type passed for 'hex_to_spline_params'. Valid argument type is HexToSplineParams.")
        args = {"input_scope" : input_scope._jsonify(),
        "hex_to_spline_params" : hex_to_spline_params._jsonify()}
        command_name = "PrimeMesh::HexToSpline/ConvertHexToSpline"
        self._model._print_beta_api_warning("convert_hex_to_spline")
        self._model._print_logs_before_command("convert_hex_to_spline", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("convert_hex_to_spline", IGAResults(model = self._model, json_data = result))
        return IGAResults(model = self._model, json_data = result)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class TrimmedSpline(CoreObject):
    """Handles creation and meshing of trimmed spline.

    Parameters
    ----------
    model : Model
        Server model to create TrimmedSpline object.
    """

    def __init__(self, model: CommunicationManager):
        """ Initialize TrimmedSpline """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::TrimmedSpline/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for TrimmedSpline. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for TrimmedSpline. """
        command_name = "PrimeMesh::TrimmedSpline/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def create_trimmed_uniform_solid_spline_by_brep_mapping(self, part_id : int, spline_params : UniformSolidSplineCreationParams) -> TrimmedSplineResults:
        """ Creates uniform solid spline and maps the CAD geometry in its parametric space.


        Parameters
        ----------
        part_id : int
            Part on which the spline is to be created.
        spline_params : UniformSolidSplineCreationParams
            Parameters used to create the spline.

        Returns
        -------
        TrimmedSplineResults
            Returns the TrimmedSplineResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = trimmedSpline.CreateTrimmedUniformSolidSplineByBrepMapping(part_id, spline_params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(spline_params, UniformSolidSplineCreationParams):
            raise TypeError("Invalid argument type passed for 'spline_params'. Valid argument type is UniformSolidSplineCreationParams.")
        args = {"part_id" : part_id,
        "spline_params" : spline_params._jsonify()}
        command_name = "PrimeMesh::TrimmedSpline/CreateTrimmedUniformSolidSplineByBrepMapping"
        self._model._print_beta_api_warning("create_trimmed_uniform_solid_spline_by_brep_mapping")
        self._model._print_logs_before_command("create_trimmed_uniform_solid_spline_by_brep_mapping", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_trimmed_uniform_solid_spline_by_brep_mapping", TrimmedSplineResults(model = self._model, json_data = result))
        return TrimmedSplineResults(model = self._model, json_data = result)

    def create_tet_mesh_on_trimmed_solid_spline(self, part_id : int, mesh_params : TetMeshSplineParams) -> TrimmedSplineResults:
        """ Creates tetrahedral mesh on trimmed solid spline.


        Parameters
        ----------
        part_id : int
            Part on which the tetrahedral meshing is performed.
        mesh_params : TetMeshSplineParams
            Parameters to configure the meshing.

        Returns
        -------
        TrimmedSplineResults
            Returns the TrimmedSplineResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = trimmedSpline.CreateTetMeshOnTrimmedSolidSpline(part_id, mesh_params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(mesh_params, TetMeshSplineParams):
            raise TypeError("Invalid argument type passed for 'mesh_params'. Valid argument type is TetMeshSplineParams.")
        args = {"part_id" : part_id,
        "mesh_params" : mesh_params._jsonify()}
        command_name = "PrimeMesh::TrimmedSpline/CreateTetMeshOnTrimmedSolidSpline"
        self._model._print_beta_api_warning("create_tet_mesh_on_trimmed_solid_spline")
        self._model._print_logs_before_command("create_tet_mesh_on_trimmed_solid_spline", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_tet_mesh_on_trimmed_solid_spline", TrimmedSplineResults(model = self._model, json_data = result))
        return TrimmedSplineResults(model = self._model, json_data = result)

    def refine_tet_mesh(self, part_id : int, refine_params : RefineTetMeshParams) -> TrimmedSplineResults:
        """ Refines tetrahedral mesh.


        Parameters
        ----------
        part_id : int
            Part on which mesh refinement is performed.
        refine_params : RefineTetMeshParams
            Parameters to configure mesh refinement.

        Returns
        -------
        TrimmedSplineResults
            Returns the TrimmedSplineResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = trimmedSpline.RefineTetMesh(part_id, refine_params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(refine_params, RefineTetMeshParams):
            raise TypeError("Invalid argument type passed for 'refine_params'. Valid argument type is RefineTetMeshParams.")
        args = {"part_id" : part_id,
        "refine_params" : refine_params._jsonify()}
        command_name = "PrimeMesh::TrimmedSpline/RefineTetMesh"
        self._model._print_beta_api_warning("refine_tet_mesh")
        self._model._print_logs_before_command("refine_tet_mesh", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("refine_tet_mesh", TrimmedSplineResults(model = self._model, json_data = result))
        return TrimmedSplineResults(model = self._model, json_data = result)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Default configuration for PyPrimeMesh."""

__all__ = [
    'ip',
    'port',
    'connection_timeout',
    'print_communicator_stats',
    'max_message_length',
    'get_examples_path',
    'get_examples_path_for_containers',
    'get_output_path',
    'get_output_path_for_containers',
]

import os

try:
    import appdirs

    USER_DATA_PATH = os.getenv(
        'PYPRIMEMESH_USER_DATA', appdirs.user_data_dir(appname='pyprimemesh', appauthor=False)
    )
except ModuleNotFoundError:
    # If appdirs is not installed, then try with tempfile.
    # NOTE: This only occurs for ADO ARM Test runs
    import tempfile

    USER_NAME = os.getenv('USERNAME', os.getenv('USER', 'pyprimemesh'))
    USER_DATA_PATH = os.getenv(
        'PYPRIMEMESH_USER_DATA', os.path.join(tempfile.gettempdir(), USER_NAME)
    )

if not os.path.exists(USER_DATA_PATH):  # pragma: no cover
    os.makedirs(USER_DATA_PATH)

EXAMPLES_PATH = os.path.join(USER_DATA_PATH, 'examples')
if not os.path.exists(EXAMPLES_PATH):  # pragma: no cover
    os.makedirs(EXAMPLES_PATH)

LOCAL_OUTDIR = os.path.join(USER_DATA_PATH, 'output')
if not os.path.exists(LOCAL_OUTDIR):  # pragma: no cover
    os.makedirs(LOCAL_OUTDIR)

CONTAINER_USER_DATA = os.getenv('PYPRIMEMESH_CONTAINER_USER_DATA ', '/data')
CONTAINER_EXAMPLES = os.path.join(CONTAINER_USER_DATA, 'examples')
CONTAINER_OUTDIR = os.path.join(CONTAINER_USER_DATA, 'output')

__DEFAULT_IP = '127.0.0.1'
__DEFAULT_PORT = 50055
__DEFAULT_CONNECTION_TIMEOUT = 20.0
__DEFAULT_COMM_LOG = False
__MAX_MESSAGE_LENGTH = 4194310

SPHINX_BUILD = bool(int(os.getenv('PYPRIMEMESH_SPHINX_BUILD', 0)))


def ip():
    """Get the default IP address used throughout the library."""
    return __DEFAULT_IP


def port():
    """Get the default port used throughout the library."""
    return __DEFAULT_PORT


def connection_timeout():
    """Get the default connection timeout used throughout the library."""
    return __DEFAULT_CONNECTION_TIMEOUT


def print_communicator_stats():
    """Get the flag to decide whether to print communicator stats(INTERNAL ONLY)."""
    return __DEFAULT_COMM_LOG


def max_message_length():
    """Get the maximum message length for a grpc channel."""
    return __MAX_MESSAGE_LENGTH


def get_examples_path():
    """Get the client-side default container path."""
    return EXAMPLES_PATH


def get_user_data_path():
    """Get the client-side default user data path."""
    return USER_DATA_PATH


def get_user_data_path_for_containers():
    """Get the user data path for containers."""
    return CONTAINER_USER_DATA


def get_examples_path_for_containers():
    """Get the server-side default container path in case of containers.

    In case of a container, the user data directory is mounted within the container image.
    """
    return CONTAINER_EXAMPLES


def get_output_path():
    """Get the client-side output directory used by containers."""
    return LOCAL_OUTDIR


def get_output_path_for_containers():
    """Get the server-side output directory used by containers."""
    return CONTAINER_OUTDIR


def get_sphinx_build():
    """Get the flag for if sphinx build is being used."""
    return SPHINX_BUILD



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Module for ToggleEdges widget."""
import os
from typing import TYPE_CHECKING

from ansys.tools.visualization_interface.backends.pyvista.widgets import PlotterWidget
from vtk import vtkPNGReader

if TYPE_CHECKING:
    from ansys.meshing.prime.graphics.plotter import PrimePlotter


class ToggleEdges(PlotterWidget):
    """Toggles the edges of the mesh objects.

    Parameters
    ----------
    prime_plotter : PrimePlotter
        Plotter to apply this widget to.
    """

    def __init__(self, prime_plotter: "PrimePlotter") -> None:
        """Initialize the widget."""
        super().__init__(prime_plotter._backend._pl.scene)
        self.prime_plotter = prime_plotter
        self._object_actors_map = self.prime_plotter._backend._object_to_actors_map
        self._button = self.prime_plotter._backend._pl.scene.add_checkbox_button_widget(
            self.callback,
            position=(5, 600),
            size=30,
            border_size=3,
            color_off="white",
            color_on="white",
        )
        self._info_actor_map = self.prime_plotter._info_actor_map

    def callback(self, state: bool) -> None:
        """Toggle the edges of the mesh objects.

        Parameters
        ----------
        state : bool
            Whether the button widget is activated.
        """
        for key, actor in self.prime_plotter._backend._pl.scene.actors.items():
            if actor in self._info_actor_map and self._info_actor_map[actor].has_mesh:
                actor.prop.show_edges = not state

    def update(self) -> None:
        """Define the configuration and representation of the button widget button."""
        show_point_vr = self._button.GetRepresentation()
        show_point_icon_file = os.path.join(os.path.dirname(__file__), "images", "show_edges.png")
        show_point_r = vtkPNGReader()
        show_point_r.SetFileName(show_point_icon_file)
        show_point_r.Update()
        image = show_point_r.GetOutput()
        show_point_vr.SetButtonTexture(0, image)
        show_point_vr.SetButtonTexture(1, image)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np


class SizeFieldType(enum.IntEnum):
    """The type of sizing field to be used to fetch element size at given location.
    """
    GEOMETRIC = 0
    """Geometric size field."""
    VOLUMETRIC = 1
    """Uses precomputed variable size in volumetric space defined by size field."""
    GEODESIC = 2
    """Uses geodesic size field."""
    CONSTANT = 3
    """Uses constant size at all locations of sizing field."""
    MESHEDGEODESIC = 7
    """Computes size field using existing surface mesh sizes and diffuses geodesically. Then uses the computed size field to remesh surfaces. Notes: The type is applicable when remeshing already meshed surfaces."""

class SolverType(enum.IntEnum):
    """Type of solver.
    """
    FLUENT = 1
    """Solver type is Fluent. Creates a group of face quality measures mostly used in Fluent."""
    MAPDL = 2
    """Solver type is MAPDL. Creates a group of face quality measures mostly used in MAPDL."""

class FaceQualityMeasure(enum.IntEnum):
    """The type of face quality measures to check face quality metrics.
    """
    SKEWNESS = 0
    """The Skewness metric ranges between 0 (worst) and 1 (best). A value of 0 indicates an equilateral cell (best) and a value of 1 indicates a completely degenerate cell (worst)."""
    SIZECHANGE = 2
    """Size Change is the maximum ratio of the area of each neighboring face element to the area of face element when the area of the face element is smaller than the neighbor. The minimum value for size change is 1."""
    ASPECTRATIO = 5
    """The Aspect Ratio metric is greater than 1. A value of 1 indicates an equilateral cell (best) and a value of 20(e.g) indicates a stretched cell (worst)."""
    WARP = 7
    """Face quality metric to check warping factor.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    ELEMENTQUALITY = 50
    """The Element Quality metric ranges between 0 (worst) and 1 (best). A value of 1 indicates a perfect cube or square (best) while a value of 0 indicates that the element has a zero or negative volume (worst)."""

class CellQualityMeasure(enum.IntEnum):
    """The type of cell quality measures to check cell quality metrics.
    """
    SKEWNESS = 0
    """The Skewness metric ranges between 0 (best) and 1 (worst). A value of 0 indicates an equilateral cell (best) and a value of 1 indicates a completely degenerate cell (worst)."""
    ASPECTRATIO = 5
    """The Aspect Ratio metric is greater than 1. A value of 1 indicates an equilateral cell (best) and a value of 20(e.g) indicates a stretched cell (worst)."""
    FLUENTASPECTRATIO = 13
    """The Fluent aspect Ratio metric is greater than 1. A value of 1 indicates an equilateral cell (best) and a value of 20(e.g) indicates a stretched cell (worst)."""
    INVERSEORTHOGONAL = 14
    """The inverse orthogonal metric ranges between 0 (best) and 1 (worst)."""
    INVERSEORTHOGONAL_V2 = 25
    """The advanced inverse orthogonal metric ranges between 0 (best) and 1 (worst)."""
    ELEMENTQUALITY = 50
    """The Element Quality metric ranges between 0 (worst) and 1 (best). A value of 1 indicates a perfect cube or square (best) while a value of 0 indicates that the element has a zero or negative volume (worst)."""

class SurfaceFeatureType(enum.IntEnum):
    """Type of face edges considered as features.
    """
    NONE = 0
    """None of face edges are considered as feature."""
    ZONEBOUNDARY = 1
    """Face edges at zone boundary are considered as feature."""
    FEATURE = 2
    """Face edges with normal angle more than threshold are considered as feature."""
    FEATUREORZONEBOUNDARY = 3
    """Face edges at zone boundary or with normal angle more than threshold are considered as feature."""
    ZONELETBOUNDARY = 4
    """Face edges at zonelet boundary are considered as feature."""
    FEATUREORZONELETBOUNDARY = 5
    """Face edges at zonelet boundary or with normal angle more than threshold are considered as feature."""

class ShellBLOffsetType(enum.IntEnum):
    """Type of offset method during ShellBL generation.
    """
    ASPECTRATIO = 0
    """Option to set ShellBL offset type as Aspect Ratio.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    LASTRATIO = 1
    """Option to set ShellBL offset type as Last Ratio.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    UNIFORM = 2
    """Option to set ShellBL offset type as Uniform.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    CURVATUREBASED = 4
    """Option to set ShellBL offset type as CurvatureBased.

    **This is a beta parameter**. **The behavior and name may change in the future**."""



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class TopoSearchField(enum.IntEnum):
    """Toposearch diagnostic field.
    """
    OVERLAPPINGTOPOFACES = 0
    """Diagnoses overlapping or partially overlapping topofaces.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    INTERSECTINGTOPOFACES = 1
    """Diagnoses intersecting topofaces.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    SELFINTERSECTINGTOPOFACES = 2
    """Diagnoses topofaces with self intersecting bounding edges.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    BROKENLOOPTOPOFACES = 3
    """Diagnoses topofaces with open or broken bounding edge loops.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    NOBOUNDARYLOOPTOPOFACES = 4
    """Diagnoses topofaces without bounding edges.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    INVALIDBOUNDARYTOPOFACES = 5
    """Diagnoses topofaces with incorrect bounding edge orientation.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    SMALLTOPOEDGES = 6
    """Diagnoses topofaces with small topoedges.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    INCORRECTBOUNDARYORIENTATIONTOPOFACES = 7
    """Diagnoses topofaces with incorrect boundary orientations.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    INCONSISTENTNORMALORIENTATIONTOPOFACES = 8
    """Diagnoses topofaces with inconsistent normal orientations.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    NUMBEROFFIELDS = 9
    """Diagnoses all topofaces.

    **This is a beta parameter**. **The behavior and name may change in the future**."""



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class VolumeMeshTool(CoreObject):
    """VolumeMeshTool allows you to check grid and improve volume mesh quality.

    VolumeMeshTool provides various volume mesh improvement algorithms.


    Parameters
    ----------
    model : Model
        Server model to create VolumeMeshTool object.
    """

    def __init__(self, model: CommunicationManager):
        """ Initialize VolumeMeshTool """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::VolumeMeshTool/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for VolumeMeshTool. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for VolumeMeshTool. """
        command_name = "PrimeMesh::VolumeMeshTool/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def assign_mesh_regions(self, target_part_id : int, target_cell_zonelets : Iterable[int], source_part_ids : Iterable[int], small_regions_volume_fraction : float) -> VolumeMeshToolResults:
        """ Assigns a region id to the cells in target cell zonelets of target part id. The region id is based on their location within source part ids.


        Parameters
        ----------
        target_part_id : int
            Id of the target part.
        target_cell_zonelets : Iterable[int]
            Ids of cell zonelets to be split into regions.
        source_part_ids : Iterable[int]
            Ids of solids used as a reference for assigning regions.
        small_regions_volume_fraction : float
            Regions with volumes smaller than a specified fraction of the total volume are merged into their largest adjacent region.
            This helps to eliminate isolated cells and thin regions from the output.

        Returns
        -------
        VolumeMeshToolResults
            Returns the VolumeMeshToolResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = volume_mesh_tool.AssignMeshRegions(target_part_id, target_cell_zonelets, source_part_ids, small_regions_volume_fraction)

        """
        if not isinstance(target_part_id, int):
            raise TypeError("Invalid argument type passed for 'target_part_id'. Valid argument type is int.")
        if not isinstance(target_cell_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'target_cell_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(source_part_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'source_part_ids'. Valid argument type is Iterable[int].")
        if not isinstance(small_regions_volume_fraction, float):
            raise TypeError("Invalid argument type passed for 'small_regions_volume_fraction'. Valid argument type is float.")
        args = {"target_part_id" : target_part_id,
        "target_cell_zonelets" : target_cell_zonelets,
        "source_part_ids" : source_part_ids,
        "small_regions_volume_fraction" : small_regions_volume_fraction}
        command_name = "PrimeMesh::VolumeMeshTool/AssignMeshRegions"
        self._model._print_beta_api_warning("assign_mesh_regions")
        self._model._print_logs_before_command("assign_mesh_regions", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("assign_mesh_regions", VolumeMeshToolResults(model = self._model, json_data = result))
        return VolumeMeshToolResults(model = self._model, json_data = result)

    def improve_by_auto_node_move(self, part_id : int, cell_zonelets : Iterable[int], boundary_zonelets : Iterable[int], params : AutoNodeMoveParams) -> VolumeMeshToolResults:
        """ Improve volume mesh by auto node move.


        Parameters
        ----------
        part_id : int
            Id of a part.
        cell_zonelets : Iterable[int]
            Ids of cell zonelets to be improved.
        boundary_zonelets : Iterable[int]
            Ids of boundary face zonelets.
        params : AutoNodeMoveParams
            Auto node move parameters.

        Returns
        -------
        VolumeMeshToolResults
            Return the VolumeMeshToolResults.


        Examples
        --------
        >>> results = volume_mesh_tool.improve_by_auto_node_move(part_id,
        >>>                                cell_zonelets,
        >>>                                boundary_zonelets,
        >>>                                prime.AutoNodeMoveParams(model =model))

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(cell_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'cell_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(boundary_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'boundary_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(params, AutoNodeMoveParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is AutoNodeMoveParams.")
        args = {"part_id" : part_id,
        "cell_zonelets" : cell_zonelets,
        "boundary_zonelets" : boundary_zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::VolumeMeshTool/ImproveByAutoNodeMove"
        self._model._print_logs_before_command("improve_by_auto_node_move", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("improve_by_auto_node_move", VolumeMeshToolResults(model = self._model, json_data = result))
        return VolumeMeshToolResults(model = self._model, json_data = result)

    def check_mesh(self, part_id : int, params : CheckMeshParams) -> CheckMeshResults:
        """ Checks mesh of a part.


        Parameters
        ----------
        part_id : int
            Id of a part.
        params : CheckMeshParams
            Parameters to check mesh.

        Returns
        -------
        CheckMeshResults
            Returns the CheckMeshResults.


        Examples
        --------
        >>> results = volume_mesh_tool.check_mesh(part_id,
        >>>                                prime.CheckMeshParams(model =model))

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(params, CheckMeshParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is CheckMeshParams.")
        args = {"part_id" : part_id,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::VolumeMeshTool/CheckMesh"
        self._model._print_logs_before_command("check_mesh", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("check_mesh", CheckMeshResults(model = self._model, json_data = result))
        return CheckMeshResults(model = self._model, json_data = result)

    def get_parts_for_points(self, points : Iterable[float], params : PartsForPointsParams) -> Iterable[int]:
        """ Finds parts enclosing the given list of points.


        Parameters
        ----------
        points : Iterable[float]
            Coordinates of points for which parts need to be found.
        params : PartsForPointsParams
            Parameters for searching parts.

        Returns
        -------
        Iterable[int]
            Returns array containing information about parts enclosing the points.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = volume_mesh_tool.get_parts_for_points([0., 0., 0.], params)

        """
        if not isinstance(points, Iterable):
            raise TypeError("Invalid argument type passed for 'points'. Valid argument type is Iterable[float].")
        if not isinstance(params, PartsForPointsParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is PartsForPointsParams.")
        args = {"points" : points,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::VolumeMeshTool/GetPartsForPoints"
        self._model._print_beta_api_warning("get_parts_for_points")
        self._model._print_logs_before_command("get_parts_for_points", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_parts_for_points")
        return result

    def copy_cell_zonelets(self, cell_zonelets : Iterable[int], target_part_id : int, params : CopyZoneletsParams) -> CopyZoneletsResults:
        """ Copy cell zonelets and face zonelets connected to the cell zonelets.


        Parameters
        ----------
        cell_zonelets : Iterable[int]
            Ids of cell zonelets to be copied.
        target_part_id : int
            Part id used to move the copied zonelets.
        params : CopyZoneletsParams
            Parameters to copy cell zonelets.

        Returns
        -------
        CopyZoneletsResults
            Returns the CopyZoneletsResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>>> results = volume_mesh_tool.copy_cell_zonelets(cell_zonelets, target_part_id = new_part.id, prime.CopyZoneletsParams(model = model))

        """
        if not isinstance(cell_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'cell_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(target_part_id, int):
            raise TypeError("Invalid argument type passed for 'target_part_id'. Valid argument type is int.")
        if not isinstance(params, CopyZoneletsParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is CopyZoneletsParams.")
        args = {"cell_zonelets" : cell_zonelets,
        "target_part_id" : target_part_id,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::VolumeMeshTool/CopyCellZonelets"
        self._model._print_beta_api_warning("copy_cell_zonelets")
        self._model._print_logs_before_command("copy_cell_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("copy_cell_zonelets", CopyZoneletsResults(model = self._model, json_data = result))
        return CopyZoneletsResults(model = self._model, json_data = result)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class ShellBLControl(CoreObject):
    """ShellBLControl allows you to generate quad mesh on face zonelets.

    ShellBLControl allows you to control generation of quad mesh on face zonelets. Controls include setting the edge scope, face scope and growth parameters.

    Parameters
    ----------
    model : Model
        Server model to create ShellBLControl object.
    id : int
        Id of the ShellBLControl.
    object_id : int
        Object id of the ShellBLControl.
    name : str
        Name of the ShellBLControl.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize ShellBLControl """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def get_growth_params(self) -> ShellBLControlGrowthParams:
        """ Gets thin ShellBL parameters for ShellBL control.


        Returns
        -------
        ShellBLControlGrowthParams
            Returns the ShellBLControlGrowthParams.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> params = shellBL_ctrl.get_growth_params()

        """
        args = {}
        command_name = "PrimeMesh::ShellBLControl/GetGrowthParams"
        self._model._print_beta_api_warning("get_growth_params")
        self._model._print_logs_before_command("get_growth_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_growth_params", ShellBLControlGrowthParams(model = self._model, json_data = result))
        return ShellBLControlGrowthParams(model = self._model, json_data = result)

    def set_growth_params(self, params : ShellBLControlGrowthParams):
        """ Sets growth parameters for ShellBL control.


        Parameters
        ----------
        params : ShellBLControlGrowthParams
            Parameters to set ShellBL growth.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = shellbl_control.set_growth_params(ShellBLControlGrowthParams(model=model))

        """
        if not isinstance(params, ShellBLControlGrowthParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is ShellBLControlGrowthParams.")
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::ShellBLControl/SetGrowthParams"
        self._model._print_beta_api_warning("set_growth_params")
        self._model._print_logs_before_command("set_growth_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_growth_params")

    def set_edge_scope(self, entities : ScopeDefinition) -> SetScopeResults:
        """ Sets the edge scope.


        Parameters
        ----------
        entities : ScopeDefinition
            Scope definition entities.

        Returns
        -------
        SetScopeResults
            Returns SetScopeResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = shellbl_control.set_edge_scope(entities)

        """
        if not isinstance(entities, ScopeDefinition):
            raise TypeError("Invalid argument type passed for 'entities'. Valid argument type is ScopeDefinition.")
        args = {"entities" : entities._jsonify()}
        command_name = "PrimeMesh::ShellBLControl/SetEdgeScope"
        self._model._print_beta_api_warning("set_edge_scope")
        self._model._print_logs_before_command("set_edge_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_edge_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def set_surface_scope(self, entities : ScopeDefinition) -> SetScopeResults:
        """ Sets the face zonelet scope.


        Parameters
        ----------
        entities : ScopeDefinition
            Scope definition entities.

        Returns
        -------
        SetScopeResults
            Returns SetScopeResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = shellbl_control.set_surface_scope(entities)

        """
        if not isinstance(entities, ScopeDefinition):
            raise TypeError("Invalid argument type passed for 'entities'. Valid argument type is ScopeDefinition.")
        args = {"entities" : entities._jsonify()}
        command_name = "PrimeMesh::ShellBLControl/SetSurfaceScope"
        self._model._print_beta_api_warning("set_surface_scope")
        self._model._print_logs_before_command("set_surface_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_surface_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def get_edge_scope(self) -> ScopeDefinition:
        """ Gets the edge scope.


        Returns
        -------
        ScopeDefinition
            Returns the ScopeDefinition.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> scope = shellbl_control.get_edge_scope()

        """
        args = {}
        command_name = "PrimeMesh::ShellBLControl/GetEdgeScope"
        self._model._print_beta_api_warning("get_edge_scope")
        self._model._print_logs_before_command("get_edge_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_edge_scope", ScopeDefinition(model = self._model, json_data = result))
        return ScopeDefinition(model = self._model, json_data = result)

    def get_surface_scope(self) -> ScopeDefinition:
        """ Gets the surface scope.


        Returns
        -------
        ScopeDefinition
            Returns the ScopeDefinition.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> scope = shellbl_control.get_surface_scope()

        """
        args = {}
        command_name = "PrimeMesh::ShellBLControl/GetSurfaceScope"
        self._model._print_beta_api_warning("get_surface_scope")
        self._model._print_logs_before_command("get_surface_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_surface_scope", ScopeDefinition(model = self._model, json_data = result))
        return ScopeDefinition(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of ShellBLControl."""
        return self._id

    @property
    def name(self):
        """ Get the name of ShellBLControl."""
        return self._name



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for imports of autogen modules."""
# isort: off
from ansys.meshing.prime.autogen.primeconfig import *
from ansys.meshing.prime.autogen.commontypes import *
from ansys.meshing.prime.autogen.commonstructs import *

# isort: on
from ansys.meshing.prime.autogen.automeshstructs import *
from ansys.meshing.prime.autogen.autoquadmesherstructs import *
from ansys.meshing.prime.autogen.collapsetoolstructs import *
from ansys.meshing.prime.autogen.connectstructs import *
from ansys.meshing.prime.autogen.controlstructs import *
from ansys.meshing.prime.autogen.deletetoolstructs import *
from ansys.meshing.prime.autogen.featureextractionstructs import *
from ansys.meshing.prime.autogen.fileiostructs import *
from ansys.meshing.prime.autogen.igastructs import *
from ansys.meshing.prime.autogen.materialpointmanagerstructs import *
from ansys.meshing.prime.autogen.meshinfostructs import *
from ansys.meshing.prime.autogen.modelstructs import *
from ansys.meshing.prime.autogen.morpherbcsstructs import *
from ansys.meshing.prime.autogen.morpherstructs import *
from ansys.meshing.prime.autogen.partstructs import *
from ansys.meshing.prime.autogen.periodiccontrolstructs import *
from ansys.meshing.prime.autogen.prismcontrolstructs import *
from ansys.meshing.prime.autogen.scaffolderstructs import *
from ansys.meshing.prime.autogen.shellblcontrolstructs import *
from ansys.meshing.prime.autogen.sizecontrolstructs import *
from ansys.meshing.prime.autogen.sizefieldstructs import *
from ansys.meshing.prime.autogen.splittoolstructs import *
from ansys.meshing.prime.autogen.surfacesearchstructs import *
from ansys.meshing.prime.autogen.surfaceutilitystructs import *
from ansys.meshing.prime.autogen.surferstructs import *
from ansys.meshing.prime.autogen.thinvolumecontrolstructs import *
from ansys.meshing.prime.autogen.topodatastructs import *
from ansys.meshing.prime.autogen.toposearchstructs import *
from ansys.meshing.prime.autogen.topoutilitystructs import *
from ansys.meshing.prime.autogen.transformstructs import *
from ansys.meshing.prime.autogen.trimmedsplinestructs import *
from ansys.meshing.prime.autogen.volumecontrolstructs import *
from ansys.meshing.prime.autogen.volumemeshtoolstructs import *
from ansys.meshing.prime.autogen.volumesearchstructs import *
from ansys.meshing.prime.autogen.volumesweeperstructs import *
from ansys.meshing.prime.autogen.vtcomposerstructs import *
from ansys.meshing.prime.autogen.wrapperstructs import *



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class ThinVolumeMeshParams(CoreObject):
    """Parameters to generate thin volume mesh. This is for internal use only.

    Parameters
    ----------
    model: Model
        Model to create a ``ThinVolumeMeshParams`` object with default parameters.
    n_layers: int, optional
        Number of thin volume layers to be generated.
    imprint_sides: bool, optional
        Checks whether to imprint quad faces on side zonelets.
    n_ignore_rings: int, optional
        Number of rings of cells at the sides to ignore when no imprint on sides. Volume fill type will be used in this buffer region at the thin volume sides to transition to thin volume mesh internally.
    json_data: dict, optional
        JSON dictionary to create a ``ThinVolumeMeshParams`` object with provided parameters.

    Examples
    --------
    >>> thin_volume_mesh_params = prime.ThinVolumeMeshParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            n_layers: int,
            imprint_sides: bool,
            n_ignore_rings: int):
        self._n_layers = n_layers
        self._imprint_sides = imprint_sides
        self._n_ignore_rings = n_ignore_rings

    def __init__(
            self,
            model: CommunicationManager=None,
            n_layers: int = None,
            imprint_sides: bool = None,
            n_ignore_rings: int = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``ThinVolumeMeshParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``ThinVolumeMeshParams`` object with default parameters.
        n_layers: int, optional
            Number of thin volume layers to be generated.
        imprint_sides: bool, optional
            Checks whether to imprint quad faces on side zonelets.
        n_ignore_rings: int, optional
            Number of rings of cells at the sides to ignore when no imprint on sides. Volume fill type will be used in this buffer region at the thin volume sides to transition to thin volume mesh internally.
        json_data: dict, optional
            JSON dictionary to create a ``ThinVolumeMeshParams`` object with provided parameters.

        Examples
        --------
        >>> thin_volume_mesh_params = prime.ThinVolumeMeshParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nLayers"] if "nLayers" in json_data else None,
                json_data["imprintSides"] if "imprintSides" in json_data else None,
                json_data["nIgnoreRings"] if "nIgnoreRings" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [n_layers, imprint_sides, n_ignore_rings])
            if all_field_specified:
                self.__initialize(
                    n_layers,
                    imprint_sides,
                    n_ignore_rings)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "ThinVolumeMeshParams")
                    json_data = param_json["ThinVolumeMeshParams"] if "ThinVolumeMeshParams" in param_json else {}
                    self.__initialize(
                        n_layers if n_layers is not None else ( ThinVolumeMeshParams._default_params["n_layers"] if "n_layers" in ThinVolumeMeshParams._default_params else (json_data["nLayers"] if "nLayers" in json_data else None)),
                        imprint_sides if imprint_sides is not None else ( ThinVolumeMeshParams._default_params["imprint_sides"] if "imprint_sides" in ThinVolumeMeshParams._default_params else (json_data["imprintSides"] if "imprintSides" in json_data else None)),
                        n_ignore_rings if n_ignore_rings is not None else ( ThinVolumeMeshParams._default_params["n_ignore_rings"] if "n_ignore_rings" in ThinVolumeMeshParams._default_params else (json_data["nIgnoreRings"] if "nIgnoreRings" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_layers: int = None,
            imprint_sides: bool = None,
            n_ignore_rings: int = None):
        """Set the default values of the ``ThinVolumeMeshParams`` object.

        Parameters
        ----------
        n_layers: int, optional
            Number of thin volume layers to be generated.
        imprint_sides: bool, optional
            Checks whether to imprint quad faces on side zonelets.
        n_ignore_rings: int, optional
            Number of rings of cells at the sides to ignore when no imprint on sides. Volume fill type will be used in this buffer region at the thin volume sides to transition to thin volume mesh internally.
        """
        args = locals()
        [ThinVolumeMeshParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``ThinVolumeMeshParams`` object.

        Examples
        --------
        >>> ThinVolumeMeshParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ThinVolumeMeshParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_layers is not None:
            json_data["nLayers"] = self._n_layers
        if self._imprint_sides is not None:
            json_data["imprintSides"] = self._imprint_sides
        if self._n_ignore_rings is not None:
            json_data["nIgnoreRings"] = self._n_ignore_rings
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_layers :  %s\nimprint_sides :  %s\nn_ignore_rings :  %s" % (self._n_layers, self._imprint_sides, self._n_ignore_rings)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_layers(self) -> int:
        """Number of thin volume layers to be generated.
        """
        return self._n_layers

    @n_layers.setter
    def n_layers(self, value: int):
        self._n_layers = value

    @property
    def imprint_sides(self) -> bool:
        """Checks whether to imprint quad faces on side zonelets.
        """
        return self._imprint_sides

    @imprint_sides.setter
    def imprint_sides(self, value: bool):
        self._imprint_sides = value

    @property
    def n_ignore_rings(self) -> int:
        """Number of rings of cells at the sides to ignore when no imprint on sides. Volume fill type will be used in this buffer region at the thin volume sides to transition to thin volume mesh internally.
        """
        return self._n_ignore_rings

    @n_ignore_rings.setter
    def n_ignore_rings(self, value: int):
        self._n_ignore_rings = value



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class VolumeSweeper(CoreObject):
    """VolumeSweeper class provide functions to volume mesh a given set of topovolumes by sweeping or stacking a set of face and edge zonelets. Provide operations to generate volume mesh using stacker technology.

    Parameters
    ----------
    model : Model
        Server model to create VolumeSweeper object.
    part_id : int
        Id of the part.
    """

    def __init__(self, model: CommunicationManager, part_id: int):
        """ Initialize VolumeSweeper  """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::VolumeSweeper/Construct"
        args = {"ModelID" : model._object_id , "PartID" : part_id, "MaxID" : -1}
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for VolumeSweeper. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for VolumeSweeper. """
        command_name = "PrimeMesh::VolumeSweeper/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def create_base_face(self, topo_volume_ids : Iterable[int], params : MeshStackerParams) -> MeshStackerResults:
        """ Creates a face at the specified origin perpendicular to the specified direction. Also, imprint model edges on the face, make necessary edge repairs, and duplicate relevant size controls on the base face.


        Parameters
        ----------
        topo_volume_ids : Iterable[int]
            Ids of volumes that need to be meshed.
        params : MeshStackerParams
            Mesh stacker parameters.

        Returns
        -------
        MeshStackerResults
            Returns the MeshStackerResults.


        Examples
        --------
        >>> results = volumesweeper.create_base_face(topo_volume_ids, params)

        """
        if not isinstance(topo_volume_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_volume_ids'. Valid argument type is Iterable[int].")
        if not isinstance(params, MeshStackerParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is MeshStackerParams.")
        args = {"topo_volume_ids" : topo_volume_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::VolumeSweeper/CreateBaseFace"
        self._model._print_logs_before_command("create_base_face", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_base_face", MeshStackerResults(model = self._model, json_data = result))
        return MeshStackerResults(model = self._model, json_data = result)

    def stack_base_face(self, base_face_ids : Iterable[int], topo_volume_ids : Iterable[int], params : MeshStackerParams) -> MeshStackerResults:
        """ Generates volume mesh stacking a meshed face layer by layer along the given direction. Calculates the stack layers using size controls and global size parameters.


        Parameters
        ----------
        base_face_ids : Iterable[int]
            Ids of base faces to be stacked
        topo_volume_ids : Iterable[int]
            Ids of volumes that need to be meshed.
        params : MeshStackerParams
            Mesh stacker parameters.

        Returns
        -------
        MeshStackerResults
            Returns the MeshStackerResults.


        Examples
        --------
        >>> results = volumesweeper.stack_base_face(base_face_ids, topo_volume_ids, params)

        """
        if not isinstance(base_face_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'base_face_ids'. Valid argument type is Iterable[int].")
        if not isinstance(topo_volume_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_volume_ids'. Valid argument type is Iterable[int].")
        if not isinstance(params, MeshStackerParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is MeshStackerParams.")
        args = {"base_face_ids" : base_face_ids,
        "topo_volume_ids" : topo_volume_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::VolumeSweeper/StackBaseFace"
        self._model._print_logs_before_command("stack_base_face", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("stack_base_face", MeshStackerResults(model = self._model, json_data = result))
        return MeshStackerResults(model = self._model, json_data = result)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class ScaffolderRepairMode(enum.IntEnum):
    """Mode of Scaffolder repair to be used.
    """
    DEFAULT = 0
    """Repairs edges using the distance tolerance parameter.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    CONSERVATIVE = 1
    """Repairs edges without considering the distance tolerance parameter.

    **This is a beta parameter**. **The behavior and name may change in the future**."""

class IntersectionMask(enum.IntEnum):
    """Scaffold parameters use intersection mask to define nature of intersection computation.
    """
    EDGEEDGE = 1
    """Performs edge to edge intersection."""
    FACEFACE = 2
    """Performs face to face intersection."""
    FACEFACEANDEDGEEDGE = 3
    """Perform face to face and edge to edge intersections."""

class EdgeMergeControl(enum.IntEnum):
    """Specifies type of edge pairs to be merged during scaffold operation.
    """
    ALLTOALL = 1
    """Allows to merge all types of edges.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    FREETOALL = 2
    """Allows to merge only free edges into all edges.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    FREETOFREE = 3
    """Allows to merge free edge into other free edge only.

    **This is a beta parameter**. **The behavior and name may change in the future**."""

class ScaffolderParams(CoreObject):
    """Parameters to control scaffold operation.

    Parameters
    ----------
    model: Model
        Model to create a ``ScaffolderParams`` object with default parameters.
    absolute_dist_tol: float, optional
        Defines the maximum gap to connect.
    repair_mode: ScaffolderRepairMode, optional
        Defines the mode to be used during repair or connect.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    size_field_type: int, optional
        Specifies the type of size field used for scaffolding.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    intersection_control_mask: IntersectionMask, optional
        Specifies the nature of intersection to be computed.
    edge_merge_control: int, optional
        Specifies type of edge pairs to be merged during scaffold operation.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    constant_mesh_size: float, optional
        Defines the constant edge mesh size to check connection.
    remove_holes_critical_radius: float, optional
        Defines the maximum radius of holes to be removed.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    remove_slivers_abs_dist_tol_ratio: float, optional
        Defines the maximum aspect ratio to remove sliver faces.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    triangles_coplanar_angle_cos: float, optional
        Lower bound for cos angle to consider coplanar faces for scaffolding.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    json_data: dict, optional
        JSON dictionary to create a ``ScaffolderParams`` object with provided parameters.

    Examples
    --------
    >>> scaffolder_params = prime.ScaffolderParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            absolute_dist_tol: float,
            repair_mode: ScaffolderRepairMode,
            size_field_type: int,
            intersection_control_mask: IntersectionMask,
            edge_merge_control: int,
            constant_mesh_size: float,
            remove_holes_critical_radius: float,
            remove_slivers_abs_dist_tol_ratio: float,
            triangles_coplanar_angle_cos: float):
        self._absolute_dist_tol = absolute_dist_tol
        self._repair_mode = ScaffolderRepairMode(repair_mode)
        self._size_field_type = size_field_type
        self._intersection_control_mask = IntersectionMask(intersection_control_mask)
        self._edge_merge_control = edge_merge_control
        self._constant_mesh_size = constant_mesh_size
        self._remove_holes_critical_radius = remove_holes_critical_radius
        self._remove_slivers_abs_dist_tol_ratio = remove_slivers_abs_dist_tol_ratio
        self._triangles_coplanar_angle_cos = triangles_coplanar_angle_cos

    def __init__(
            self,
            model: CommunicationManager=None,
            absolute_dist_tol: float = None,
            repair_mode: ScaffolderRepairMode = None,
            size_field_type: int = None,
            intersection_control_mask: IntersectionMask = None,
            edge_merge_control: int = None,
            constant_mesh_size: float = None,
            remove_holes_critical_radius: float = None,
            remove_slivers_abs_dist_tol_ratio: float = None,
            triangles_coplanar_angle_cos: float = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``ScaffolderParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``ScaffolderParams`` object with default parameters.
        absolute_dist_tol: float, optional
            Defines the maximum gap to connect.
        repair_mode: ScaffolderRepairMode, optional
            Defines the mode to be used during repair or connect.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        size_field_type: int, optional
            Specifies the type of size field used for scaffolding.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        intersection_control_mask: IntersectionMask, optional
            Specifies the nature of intersection to be computed.
        edge_merge_control: int, optional
            Specifies type of edge pairs to be merged during scaffold operation.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        constant_mesh_size: float, optional
            Defines the constant edge mesh size to check connection.
        remove_holes_critical_radius: float, optional
            Defines the maximum radius of holes to be removed.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        remove_slivers_abs_dist_tol_ratio: float, optional
            Defines the maximum aspect ratio to remove sliver faces.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        triangles_coplanar_angle_cos: float, optional
            Lower bound for cos angle to consider coplanar faces for scaffolding.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        json_data: dict, optional
            JSON dictionary to create a ``ScaffolderParams`` object with provided parameters.

        Examples
        --------
        >>> scaffolder_params = prime.ScaffolderParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["absoluteDistTol"] if "absoluteDistTol" in json_data else None,
                ScaffolderRepairMode(json_data["repairMode"] if "repairMode" in json_data else None),
                json_data["sizeFieldType"] if "sizeFieldType" in json_data else None,
                IntersectionMask(json_data["intersectionControlMask"] if "intersectionControlMask" in json_data else None),
                json_data["edgeMergeControl"] if "edgeMergeControl" in json_data else None,
                json_data["constantMeshSize"] if "constantMeshSize" in json_data else None,
                json_data["removeHolesCriticalRadius"] if "removeHolesCriticalRadius" in json_data else None,
                json_data["removeSliversAbsDistTolRatio"] if "removeSliversAbsDistTolRatio" in json_data else None,
                json_data["trianglesCoplanarAngleCos"] if "trianglesCoplanarAngleCos" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [absolute_dist_tol, repair_mode, size_field_type, intersection_control_mask, edge_merge_control, constant_mesh_size, remove_holes_critical_radius, remove_slivers_abs_dist_tol_ratio, triangles_coplanar_angle_cos])
            if all_field_specified:
                self.__initialize(
                    absolute_dist_tol,
                    repair_mode,
                    size_field_type,
                    intersection_control_mask,
                    edge_merge_control,
                    constant_mesh_size,
                    remove_holes_critical_radius,
                    remove_slivers_abs_dist_tol_ratio,
                    triangles_coplanar_angle_cos)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "ScaffolderParams")
                    json_data = param_json["ScaffolderParams"] if "ScaffolderParams" in param_json else {}
                    self.__initialize(
                        absolute_dist_tol if absolute_dist_tol is not None else ( ScaffolderParams._default_params["absolute_dist_tol"] if "absolute_dist_tol" in ScaffolderParams._default_params else (json_data["absoluteDistTol"] if "absoluteDistTol" in json_data else None)),
                        repair_mode if repair_mode is not None else ( ScaffolderParams._default_params["repair_mode"] if "repair_mode" in ScaffolderParams._default_params else ScaffolderRepairMode(json_data["repairMode"] if "repairMode" in json_data else None)),
                        size_field_type if size_field_type is not None else ( ScaffolderParams._default_params["size_field_type"] if "size_field_type" in ScaffolderParams._default_params else (json_data["sizeFieldType"] if "sizeFieldType" in json_data else None)),
                        intersection_control_mask if intersection_control_mask is not None else ( ScaffolderParams._default_params["intersection_control_mask"] if "intersection_control_mask" in ScaffolderParams._default_params else IntersectionMask(json_data["intersectionControlMask"] if "intersectionControlMask" in json_data else None)),
                        edge_merge_control if edge_merge_control is not None else ( ScaffolderParams._default_params["edge_merge_control"] if "edge_merge_control" in ScaffolderParams._default_params else (json_data["edgeMergeControl"] if "edgeMergeControl" in json_data else None)),
                        constant_mesh_size if constant_mesh_size is not None else ( ScaffolderParams._default_params["constant_mesh_size"] if "constant_mesh_size" in ScaffolderParams._default_params else (json_data["constantMeshSize"] if "constantMeshSize" in json_data else None)),
                        remove_holes_critical_radius if remove_holes_critical_radius is not None else ( ScaffolderParams._default_params["remove_holes_critical_radius"] if "remove_holes_critical_radius" in ScaffolderParams._default_params else (json_data["removeHolesCriticalRadius"] if "removeHolesCriticalRadius" in json_data else None)),
                        remove_slivers_abs_dist_tol_ratio if remove_slivers_abs_dist_tol_ratio is not None else ( ScaffolderParams._default_params["remove_slivers_abs_dist_tol_ratio"] if "remove_slivers_abs_dist_tol_ratio" in ScaffolderParams._default_params else (json_data["removeSliversAbsDistTolRatio"] if "removeSliversAbsDistTolRatio" in json_data else None)),
                        triangles_coplanar_angle_cos if triangles_coplanar_angle_cos is not None else ( ScaffolderParams._default_params["triangles_coplanar_angle_cos"] if "triangles_coplanar_angle_cos" in ScaffolderParams._default_params else (json_data["trianglesCoplanarAngleCos"] if "trianglesCoplanarAngleCos" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            absolute_dist_tol: float = None,
            repair_mode: ScaffolderRepairMode = None,
            size_field_type: int = None,
            intersection_control_mask: IntersectionMask = None,
            edge_merge_control: int = None,
            constant_mesh_size: float = None,
            remove_holes_critical_radius: float = None,
            remove_slivers_abs_dist_tol_ratio: float = None,
            triangles_coplanar_angle_cos: float = None):
        """Set the default values of the ``ScaffolderParams`` object.

        Parameters
        ----------
        absolute_dist_tol: float, optional
            Defines the maximum gap to connect.
        repair_mode: ScaffolderRepairMode, optional
            Defines the mode to be used during repair or connect.
        size_field_type: int, optional
            Specifies the type of size field used for scaffolding.
        intersection_control_mask: IntersectionMask, optional
            Specifies the nature of intersection to be computed.
        edge_merge_control: int, optional
            Specifies type of edge pairs to be merged during scaffold operation.
        constant_mesh_size: float, optional
            Defines the constant edge mesh size to check connection.
        remove_holes_critical_radius: float, optional
            Defines the maximum radius of holes to be removed.
        remove_slivers_abs_dist_tol_ratio: float, optional
            Defines the maximum aspect ratio to remove sliver faces.
        triangles_coplanar_angle_cos: float, optional
            Lower bound for cos angle to consider coplanar faces for scaffolding.
        """
        args = locals()
        [ScaffolderParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``ScaffolderParams`` object.

        Examples
        --------
        >>> ScaffolderParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ScaffolderParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._absolute_dist_tol is not None:
            json_data["absoluteDistTol"] = self._absolute_dist_tol
        if self._repair_mode is not None:
            json_data["repairMode"] = self._repair_mode
        if self._size_field_type is not None:
            json_data["sizeFieldType"] = self._size_field_type
        if self._intersection_control_mask is not None:
            json_data["intersectionControlMask"] = self._intersection_control_mask
        if self._edge_merge_control is not None:
            json_data["edgeMergeControl"] = self._edge_merge_control
        if self._constant_mesh_size is not None:
            json_data["constantMeshSize"] = self._constant_mesh_size
        if self._remove_holes_critical_radius is not None:
            json_data["removeHolesCriticalRadius"] = self._remove_holes_critical_radius
        if self._remove_slivers_abs_dist_tol_ratio is not None:
            json_data["removeSliversAbsDistTolRatio"] = self._remove_slivers_abs_dist_tol_ratio
        if self._triangles_coplanar_angle_cos is not None:
            json_data["trianglesCoplanarAngleCos"] = self._triangles_coplanar_angle_cos
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "absolute_dist_tol :  %s\nrepair_mode :  %s\nsize_field_type :  %s\nintersection_control_mask :  %s\nedge_merge_control :  %s\nconstant_mesh_size :  %s\nremove_holes_critical_radius :  %s\nremove_slivers_abs_dist_tol_ratio :  %s\ntriangles_coplanar_angle_cos :  %s" % (self._absolute_dist_tol, self._repair_mode, self._size_field_type, self._intersection_control_mask, self._edge_merge_control, self._constant_mesh_size, self._remove_holes_critical_radius, self._remove_slivers_abs_dist_tol_ratio, self._triangles_coplanar_angle_cos)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def absolute_dist_tol(self) -> float:
        """Defines the maximum gap to connect.
        """
        return self._absolute_dist_tol

    @absolute_dist_tol.setter
    def absolute_dist_tol(self, value: float):
        self._absolute_dist_tol = value

    @property
    def repair_mode(self) -> ScaffolderRepairMode:
        """Defines the mode to be used during repair or connect.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._repair_mode

    @repair_mode.setter
    def repair_mode(self, value: ScaffolderRepairMode):
        self._repair_mode = value

    @property
    def size_field_type(self) -> int:
        """Specifies the type of size field used for scaffolding.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._size_field_type

    @size_field_type.setter
    def size_field_type(self, value: int):
        self._size_field_type = value

    @property
    def intersection_control_mask(self) -> IntersectionMask:
        """Specifies the nature of intersection to be computed.
        """
        return self._intersection_control_mask

    @intersection_control_mask.setter
    def intersection_control_mask(self, value: IntersectionMask):
        self._intersection_control_mask = value

    @property
    def edge_merge_control(self) -> int:
        """Specifies type of edge pairs to be merged during scaffold operation.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._edge_merge_control

    @edge_merge_control.setter
    def edge_merge_control(self, value: int):
        self._edge_merge_control = value

    @property
    def constant_mesh_size(self) -> float:
        """Defines the constant edge mesh size to check connection.
        """
        return self._constant_mesh_size

    @constant_mesh_size.setter
    def constant_mesh_size(self, value: float):
        self._constant_mesh_size = value

    @property
    def remove_holes_critical_radius(self) -> float:
        """Defines the maximum radius of holes to be removed.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._remove_holes_critical_radius

    @remove_holes_critical_radius.setter
    def remove_holes_critical_radius(self, value: float):
        self._remove_holes_critical_radius = value

    @property
    def remove_slivers_abs_dist_tol_ratio(self) -> float:
        """Defines the maximum aspect ratio to remove sliver faces.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._remove_slivers_abs_dist_tol_ratio

    @remove_slivers_abs_dist_tol_ratio.setter
    def remove_slivers_abs_dist_tol_ratio(self, value: float):
        self._remove_slivers_abs_dist_tol_ratio = value

    @property
    def triangles_coplanar_angle_cos(self) -> float:
        """Lower bound for cos angle to consider coplanar faces for scaffolding.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._triangles_coplanar_angle_cos

    @triangles_coplanar_angle_cos.setter
    def triangles_coplanar_angle_cos(self, value: float):
        self._triangles_coplanar_angle_cos = value

class VolumetricScaffolderParams(CoreObject):
    """Parameters to control delete shadowed topofaces operation.

    Parameters
    ----------
    model: Model
        Model to create a ``VolumetricScaffolderParams`` object with default parameters.
    absolute_dist_tol: float, optional
        Specify distance tolerance between overlapping faces.
    only_check_exact_overlaps: bool, optional
        Check only for fully overlapping topofaces when true.
    json_data: dict, optional
        JSON dictionary to create a ``VolumetricScaffolderParams`` object with provided parameters.

    Examples
    --------
    >>> volumetric_scaffolder_params = prime.VolumetricScaffolderParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            absolute_dist_tol: float,
            only_check_exact_overlaps: bool):
        self._absolute_dist_tol = absolute_dist_tol
        self._only_check_exact_overlaps = only_check_exact_overlaps

    def __init__(
            self,
            model: CommunicationManager=None,
            absolute_dist_tol: float = None,
            only_check_exact_overlaps: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``VolumetricScaffolderParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``VolumetricScaffolderParams`` object with default parameters.
        absolute_dist_tol: float, optional
            Specify distance tolerance between overlapping faces.
        only_check_exact_overlaps: bool, optional
            Check only for fully overlapping topofaces when true.
        json_data: dict, optional
            JSON dictionary to create a ``VolumetricScaffolderParams`` object with provided parameters.

        Examples
        --------
        >>> volumetric_scaffolder_params = prime.VolumetricScaffolderParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["absoluteDistTol"] if "absoluteDistTol" in json_data else None,
                json_data["onlyCheckExactOverlaps"] if "onlyCheckExactOverlaps" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [absolute_dist_tol, only_check_exact_overlaps])
            if all_field_specified:
                self.__initialize(
                    absolute_dist_tol,
                    only_check_exact_overlaps)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumetricScaffolderParams")
                    json_data = param_json["VolumetricScaffolderParams"] if "VolumetricScaffolderParams" in param_json else {}
                    self.__initialize(
                        absolute_dist_tol if absolute_dist_tol is not None else ( VolumetricScaffolderParams._default_params["absolute_dist_tol"] if "absolute_dist_tol" in VolumetricScaffolderParams._default_params else (json_data["absoluteDistTol"] if "absoluteDistTol" in json_data else None)),
                        only_check_exact_overlaps if only_check_exact_overlaps is not None else ( VolumetricScaffolderParams._default_params["only_check_exact_overlaps"] if "only_check_exact_overlaps" in VolumetricScaffolderParams._default_params else (json_data["onlyCheckExactOverlaps"] if "onlyCheckExactOverlaps" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            absolute_dist_tol: float = None,
            only_check_exact_overlaps: bool = None):
        """Set the default values of the ``VolumetricScaffolderParams`` object.

        Parameters
        ----------
        absolute_dist_tol: float, optional
            Specify distance tolerance between overlapping faces.
        only_check_exact_overlaps: bool, optional
            Check only for fully overlapping topofaces when true.
        """
        args = locals()
        [VolumetricScaffolderParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``VolumetricScaffolderParams`` object.

        Examples
        --------
        >>> VolumetricScaffolderParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumetricScaffolderParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._absolute_dist_tol is not None:
            json_data["absoluteDistTol"] = self._absolute_dist_tol
        if self._only_check_exact_overlaps is not None:
            json_data["onlyCheckExactOverlaps"] = self._only_check_exact_overlaps
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "absolute_dist_tol :  %s\nonly_check_exact_overlaps :  %s" % (self._absolute_dist_tol, self._only_check_exact_overlaps)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def absolute_dist_tol(self) -> float:
        """Specify distance tolerance between overlapping faces.
        """
        return self._absolute_dist_tol

    @absolute_dist_tol.setter
    def absolute_dist_tol(self, value: float):
        self._absolute_dist_tol = value

    @property
    def only_check_exact_overlaps(self) -> bool:
        """Check only for fully overlapping topofaces when true.
        """
        return self._only_check_exact_overlaps

    @only_check_exact_overlaps.setter
    def only_check_exact_overlaps(self, value: bool):
        self._only_check_exact_overlaps = value

class ScaffolderResults(CoreObject):
    """Results structure associated to scaffold operation.

    Parameters
    ----------
    model: Model
        Model to create a ``ScaffolderResults`` object with default parameters.
    n_incomplete_topo_faces: int, optional
        Number of topofaces failed in scaffold operation.
    error_code: ErrorCode, optional
        Error code associated with scaffold operation.
    json_data: dict, optional
        JSON dictionary to create a ``ScaffolderResults`` object with provided parameters.

    Examples
    --------
    >>> scaffolder_results = prime.ScaffolderResults(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            n_incomplete_topo_faces: int,
            error_code: ErrorCode):
        self._n_incomplete_topo_faces = n_incomplete_topo_faces
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            n_incomplete_topo_faces: int = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``ScaffolderResults`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``ScaffolderResults`` object with default parameters.
        n_incomplete_topo_faces: int, optional
            Number of topofaces failed in scaffold operation.
        error_code: ErrorCode, optional
            Error code associated with scaffold operation.
        json_data: dict, optional
            JSON dictionary to create a ``ScaffolderResults`` object with provided parameters.

        Examples
        --------
        >>> scaffolder_results = prime.ScaffolderResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nIncompleteTopoFaces"] if "nIncompleteTopoFaces" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [n_incomplete_topo_faces, error_code])
            if all_field_specified:
                self.__initialize(
                    n_incomplete_topo_faces,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "ScaffolderResults")
                    json_data = param_json["ScaffolderResults"] if "ScaffolderResults" in param_json else {}
                    self.__initialize(
                        n_incomplete_topo_faces if n_incomplete_topo_faces is not None else ( ScaffolderResults._default_params["n_incomplete_topo_faces"] if "n_incomplete_topo_faces" in ScaffolderResults._default_params else (json_data["nIncompleteTopoFaces"] if "nIncompleteTopoFaces" in json_data else None)),
                        error_code if error_code is not None else ( ScaffolderResults._default_params["error_code"] if "error_code" in ScaffolderResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_incomplete_topo_faces: int = None,
            error_code: ErrorCode = None):
        """Set the default values of the ``ScaffolderResults`` object.

        Parameters
        ----------
        n_incomplete_topo_faces: int, optional
            Number of topofaces failed in scaffold operation.
        error_code: ErrorCode, optional
            Error code associated with scaffold operation.
        """
        args = locals()
        [ScaffolderResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``ScaffolderResults`` object.

        Examples
        --------
        >>> ScaffolderResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ScaffolderResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_incomplete_topo_faces is not None:
            json_data["nIncompleteTopoFaces"] = self._n_incomplete_topo_faces
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_incomplete_topo_faces :  %s\nerror_code :  %s" % (self._n_incomplete_topo_faces, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_incomplete_topo_faces(self) -> int:
        """Number of topofaces failed in scaffold operation.
        """
        return self._n_incomplete_topo_faces

    @n_incomplete_topo_faces.setter
    def n_incomplete_topo_faces(self, value: int):
        self._n_incomplete_topo_faces = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with scaffold operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class ScaffolderSplitResults(CoreObject):
    """Result structure associated to split topofaces operation.

    Parameters
    ----------
    model: Model
        Model to create a ``ScaffolderSplitResults`` object with default parameters.
    new_faces: Iterable[int], optional
        Topofaces created after split operation.
    error_code: ErrorCode, optional
        Error code associated with split topofaces operation.
    json_data: dict, optional
        JSON dictionary to create a ``ScaffolderSplitResults`` object with provided parameters.

    Examples
    --------
    >>> scaffolder_split_results = prime.ScaffolderSplitResults(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            new_faces: Iterable[int],
            error_code: ErrorCode):
        self._new_faces = new_faces if isinstance(new_faces, np.ndarray) else np.array(new_faces, dtype=np.int32) if new_faces is not None else None
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            new_faces: Iterable[int] = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``ScaffolderSplitResults`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``ScaffolderSplitResults`` object with default parameters.
        new_faces: Iterable[int], optional
            Topofaces created after split operation.
        error_code: ErrorCode, optional
            Error code associated with split topofaces operation.
        json_data: dict, optional
            JSON dictionary to create a ``ScaffolderSplitResults`` object with provided parameters.

        Examples
        --------
        >>> scaffolder_split_results = prime.ScaffolderSplitResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["newFaces"] if "newFaces" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [new_faces, error_code])
            if all_field_specified:
                self.__initialize(
                    new_faces,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "ScaffolderSplitResults")
                    json_data = param_json["ScaffolderSplitResults"] if "ScaffolderSplitResults" in param_json else {}
                    self.__initialize(
                        new_faces if new_faces is not None else ( ScaffolderSplitResults._default_params["new_faces"] if "new_faces" in ScaffolderSplitResults._default_params else (json_data["newFaces"] if "newFaces" in json_data else None)),
                        error_code if error_code is not None else ( ScaffolderSplitResults._default_params["error_code"] if "error_code" in ScaffolderSplitResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            new_faces: Iterable[int] = None,
            error_code: ErrorCode = None):
        """Set the default values of the ``ScaffolderSplitResults`` object.

        Parameters
        ----------
        new_faces: Iterable[int], optional
            Topofaces created after split operation.
        error_code: ErrorCode, optional
            Error code associated with split topofaces operation.
        """
        args = locals()
        [ScaffolderSplitResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``ScaffolderSplitResults`` object.

        Examples
        --------
        >>> ScaffolderSplitResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ScaffolderSplitResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._new_faces is not None:
            json_data["newFaces"] = self._new_faces
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "new_faces :  %s\nerror_code :  %s" % (self._new_faces, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def new_faces(self) -> Iterable[int]:
        """Topofaces created after split operation.
        """
        return self._new_faces

    @new_faces.setter
    def new_faces(self, value: Iterable[int]):
        self._new_faces = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with split topofaces operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class ScaffolderMergeResults(CoreObject):
    """Result structure associated with merge overlapping topofaces and delete shadowed topofaces operations.

    Parameters
    ----------
    model: Model
        Model to create a ``ScaffolderMergeResults`` object with default parameters.
    n_merged: int, optional
        Number of merged topofaces.
    error_code: ErrorCode, optional
        Error code associated with merge overlapping topofaces operation.
    json_data: dict, optional
        JSON dictionary to create a ``ScaffolderMergeResults`` object with provided parameters.

    Examples
    --------
    >>> scaffolder_merge_results = prime.ScaffolderMergeResults(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            n_merged: int,
            error_code: ErrorCode):
        self._n_merged = n_merged
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            n_merged: int = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``ScaffolderMergeResults`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``ScaffolderMergeResults`` object with default parameters.
        n_merged: int, optional
            Number of merged topofaces.
        error_code: ErrorCode, optional
            Error code associated with merge overlapping topofaces operation.
        json_data: dict, optional
            JSON dictionary to create a ``ScaffolderMergeResults`` object with provided parameters.

        Examples
        --------
        >>> scaffolder_merge_results = prime.ScaffolderMergeResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nMerged"] if "nMerged" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [n_merged, error_code])
            if all_field_specified:
                self.__initialize(
                    n_merged,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "ScaffolderMergeResults")
                    json_data = param_json["ScaffolderMergeResults"] if "ScaffolderMergeResults" in param_json else {}
                    self.__initialize(
                        n_merged if n_merged is not None else ( ScaffolderMergeResults._default_params["n_merged"] if "n_merged" in ScaffolderMergeResults._default_params else (json_data["nMerged"] if "nMerged" in json_data else None)),
                        error_code if error_code is not None else ( ScaffolderMergeResults._default_params["error_code"] if "error_code" in ScaffolderMergeResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_merged: int = None,
            error_code: ErrorCode = None):
        """Set the default values of the ``ScaffolderMergeResults`` object.

        Parameters
        ----------
        n_merged: int, optional
            Number of merged topofaces.
        error_code: ErrorCode, optional
            Error code associated with merge overlapping topofaces operation.
        """
        args = locals()
        [ScaffolderMergeResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``ScaffolderMergeResults`` object.

        Examples
        --------
        >>> ScaffolderMergeResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ScaffolderMergeResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_merged is not None:
            json_data["nMerged"] = self._n_merged
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_merged :  %s\nerror_code :  %s" % (self._n_merged, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_merged(self) -> int:
        """Number of merged topofaces.
        """
        return self._n_merged

    @n_merged.setter
    def n_merged(self, value: int):
        self._n_merged = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with merge overlapping topofaces operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value
